window.Gleam.initSearch([{"doc":"gemqtt","title":"gemqtt","content":"# gemqtt\n\n[![Package Version](https://img.shields.io/hexpm/v/gemqtt)](https://hex.pm/packages/gemqtt)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gemqtt/)\n\ngemqtt provides an MQTT client for Gleam projects running on the BEAM.  It does\nso by wrapping the [emqx/emqtt] library written in Erlang.\n\n## NOTE\n\nAt the time of writing (March 19th, 2024), this package is stuck on emqtt\nversion 1.2 due to a [dependency\nissue](https://github.com/jhillyerd/gemqtt/issues/21). That version of the\nunderlying [emqx/emqtt] library only works with Erlang OTP version 25.x, not\n26+.\n\n## Status\n\nThis package is a work in progress. If it is missing an emqtt feature you need,\nplease send a PR.\n\n### General features\n\n- [x] Connect to unencrypted MQTT servers over TCP\n- [x] Connect to TLS encrypted MQTT servers over TCP\n- [ ] Connect to unencrypted MQTT servers over websocket\n- [ ] Connect to TLS encrypted MQTT servers over websocket\n- [x] Supports [emqtt properties] for connect, publish, and subscribe\n\n### Connect options\n\n- [x] User + password authentication\n- [x] Message auto acknowledgement\n- [x] Clean start\n- [x] Client ID\n- [x] Connect timeout\n- [x] Erlang server name\n- [x] Owner PID (for disconnect notifications)\n- [x] Port number\n- [x] TLS enable + raw options\n- [ ] TCP options\n- [ ] Websocket path\n- [ ] Bridge mode\n- [ ] Proto version\n- [ ] Keep alive\n- [ ] Max in-flight\n- [ ] Retry interval\n- [ ] Will topic\n- [ ] Will payload\n- [ ] Will retain\n- [ ] Will QoS\n- [ ] Will properties\n- [ ] Ack timeout\n- [ ] Force ping\n\n### Publish options\n\n- [x] QoS\n- [x] Retain\n\n### Subscribe options\n\n- [x] Local Echo\n- [x] QoS\n- [x] Retain as published\n- [x] Retain handling\n\n## Example Usage\n\n```sh\ngleam add gleam_erlang\ngleam add gemqtt\n```\n```gleam\nimport gemqtt\nimport gemqtt/publisher\nimport gemqtt/subscriber\nimport gleam/bit_array\nimport gleam/erlang/process\nimport gleam/io\n\npub fn main() {\n  let topic = &quot;gemqtt/readme/example&quot;\n\n  // Create a client and connect to the test server.\n  let assert Ok(client) =\n    gemqtt.new(&quot;test.mosquitto.org&quot;)\n    |&gt; gemqtt.start_link\n  let assert Ok(_) = gemqtt.connect(client)\n\n  io.println(&quot;Connected.&quot;)\n\n  // Subscribe to messages from the topic.\n  let assert Ok(_) =\n    client\n    |&gt; subscriber.new\n    |&gt; subscriber.add(topics: [topic])\n\n  io.println(&quot;Subscribed.&quot;)\n\n  // Publish a test message to the topic.\n  let assert Ok(_) =\n    publisher.new(client, topic)\n    |&gt; publisher.publish(bit_array.from_string(&quot;Hello from Gleam!&quot;))\n\n  io.println(&quot;Sent message.&quot;)\n\n  // Attempt to receive a message from the topic.\n  let assert Ok(got_msg) =\n    process.new_selector()\n    |&gt; subscriber.selecting_mqtt_messages(Ok)\n    |&gt; process.select_forever\n\n  io.println(&quot;Received message:&quot;)\n  io.debug(got_msg)\n\n  let assert Ok(_) = subscriber.remove(client, [topic])\n  let assert Ok(_) = gemqtt.disconnect(client)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gemqtt&gt;.\n\n## Development\n\n```sh\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n\n\n[emqx/emqtt]:       https://github.com/emqx/emqtt\n[emqtt properties]: https://github.com/emqx/emqtt?tab=readme-ov-file#properties\n","url":"index.html"},{"doc":"gemqtt","title":"Client","content":"pub opaque type Client\n Client holds the process maintaining the MQTT connection, and is required\n to publish and subsribe to messages.\n\n\nSynonyms:\ngemqtt.Client\ngemqtt Client","url":"gemqtt.html#Client"},{"doc":"gemqtt","title":"Error","content":"pub type Error {\n  AlreadyStarted(process.Pid)\n  BadProperty(Option(Atom))\n  Noproc\n  Closed\n  Timeout\n  Eaddrinuse\n  Eaddrnotavail\n  Eafnosupport\n  Ealready\n  Econnaborted\n  Econnrefused\n  Econnreset\n  Edestaddrreq\n  Ehostdown\n  Ehostunreach\n  Einprogress\n  Eisconn\n  Emsgsize\n  Enetdown\n  Enetunreach\n  Enopkg\n  Enoprotoopt\n  Enotconn\n  Enotty\n  Enotsock\n  Eproto\n  Eprotonosupport\n  Eprototype\n  Esocktnosupport\n  Etimedout\n  Ewouldblock\n  Exbadport\n  Exbadseq\n  Nxdomain\n  AccessDenied\n  BadCertificate\n  BadCertificateHashValue\n  BadCertificateStatusResponse\n  BadRecordMac\n  CertificateExpired\n  CertificateRevoked\n  CertificateUnknown\n  CertificateUnobtainable\n  CloseNotify\n  DecodeError\n  DecryptError\n  ExportRestriction\n  HandshakeFailure\n  IllegalParameter\n  InappropriateFallback\n  InsufficientSecurity\n  InternalError\n  NoApplicationProtocol\n  NoRenegotiation\n  ProtocolVersion\n  RecordOverflow\n  UnexpectedMessage\n  UnknownCa\n  UnknownPskIdentity\n  UnrecognizedName\n  UnsupportedCertificate\n  UnsupportedExtension\n  UserCanceled\n}\n Errors that can occur when working with MQTT connections.\n\nAlreadyStarted(process.Pid)\n\n\nBadProperty(Option(Atom))\n\n\nNoproc\n\n\nClosed\n\n\nTimeout\n\n\nEaddrinuse\n\n\nEaddrnotavail\n\n\nEafnosupport\n\n\nEalready\n\n\nEconnaborted\n\n\nEconnrefused\n\n\nEconnreset\n\n\nEdestaddrreq\n\n\nEhostdown\n\n\nEhostunreach\n\n\nEinprogress\n\n\nEisconn\n\n\nEmsgsize\n\n\nEnetdown\n\n\nEnetunreach\n\n\nEnopkg\n\n\nEnoprotoopt\n\n\nEnotconn\n\n\nEnotty\n\n\nEnotsock\n\n\nEproto\n\n\nEprotonosupport\n\n\nEprototype\n\n\nEsocktnosupport\n\n\nEtimedout\n\n\nEwouldblock\n\n\nExbadport\n\n\nExbadseq\n\n\nNxdomain\n\n\nAccessDenied\n\n\nBadCertificate\n\n\nBadCertificateHashValue\n\n\nBadCertificateStatusResponse\n\n\nBadRecordMac\n\n\nCertificateExpired\n\n\nCertificateRevoked\n\n\nCertificateUnknown\n\n\nCertificateUnobtainable\n\n\nCloseNotify\n\n\nDecodeError\n\n\nDecryptError\n\n\nExportRestriction\n\n\nHandshakeFailure\n\n\nIllegalParameter\n\n\nInappropriateFallback\n\n\nInsufficientSecurity\n\n\nInternalError\n\n\nNoApplicationProtocol\n\n\nNoRenegotiation\n\n\nProtocolVersion\n\n\nRecordOverflow\n\n\nUnexpectedMessage\n\n\nUnknownCa\n\n\nUnknownPskIdentity\n\n\nUnrecognizedName\n\n\nUnsupportedCertificate\n\n\nUnsupportedExtension\n\n\nUserCanceled\n\n\nSynonyms:\ngemqtt.Error\ngemqtt Error","url":"gemqtt.html#Error"},{"doc":"gemqtt","title":"Options","content":"pub opaque type Options\n MQTT connection option holder. Use `new` to create it, followed by the set\n functions in this module to configure it, finally passing it to\n `start_link` to create a Client.\n\n\nSynonyms:\ngemqtt.Options\ngemqtt Options","url":"gemqtt.html#Options"},{"doc":"gemqtt","title":"Properties","content":"pub type Properties {\n  Properties(Dict(Atom, Dynamic))\n}\n _emqtt_ properties holder. Properties can control advanced MQTT protocol\n features; see the documentation on `set_property` in each of the `gemqtt`,\n `publisher`, and `subscriber` modules.\n\n There is also information in the [emqtt\n properties](https://github.com/emqx/emqtt?tab=readme-ov-file#properties)\n docs for more information.\n\nProperties(Dict(Atom, Dynamic))\n\n\nSynonyms:\ngemqtt.Properties\ngemqtt Properties","url":"gemqtt.html#Properties"},{"doc":"gemqtt","title":"Qos","content":"pub type Qos {\n  AtMostOnce\n  AtLeastOnce\n  ExactlyOnce\n}\n MQTT Quality of Service level, controlling how many times a message\n may be delivered.\n\nAtMostOnce\n\n\nAtLeastOnce\n\n\nExactlyOnce\n\n\nSynonyms:\ngemqtt.Qos\ngemqtt Qos","url":"gemqtt.html#Qos"},{"doc":"gemqtt","title":"connect","content":"pub fn connect(client: Client) -&gt; Result(Nil, Error)\n Connects to the configured MQTT server.\n\nSynonyms:\ngemqtt.connect\ngemqtt connect","url":"gemqtt.html#connect"},{"doc":"gemqtt","title":"disconnect","content":"pub fn disconnect(client: Client) -&gt; Result(Nil, Error)\n Sends a DISCONNECT packet to the server, and closes the connection. The\n Client process will then exit.\n\nSynonyms:\ngemqtt.disconnect\ngemqtt disconnect","url":"gemqtt.html#disconnect"},{"doc":"gemqtt","title":"new","content":"pub fn new(host: String) -&gt; Options\n Creates a new `Options` holder for the specified MQTT server.  Do not\n include a port number in the `host` string, instead use `set_port`.\n\nSynonyms:\ngemqtt.new\ngemqtt new","url":"gemqtt.html#new"},{"doc":"gemqtt","title":"pid_of","content":"pub fn pid_of(client: Client) -&gt; Pid\n Returns the Erlang Pid of the provided Client.\n\nSynonyms:\ngemqtt.pid_of\ngemqtt pid_of","url":"gemqtt.html#pid_of"},{"doc":"gemqtt","title":"set_auth","content":"pub fn set_auth(\n  opts: Options,\n  username: String,\n  password: String,\n) -&gt; Options\n Specifies authentication credentials to use with the MQTT server.\n\nSynonyms:\ngemqtt.set_auth\ngemqtt set_auth","url":"gemqtt.html#set_auth"},{"doc":"gemqtt","title":"set_auto_ack","content":"pub fn set_auto_ack(opts: Options, ack: Bool) -&gt; Options\n If true (the default), the client process will automatically send an\n acknowledgement (ie PUBACK) when it receives a packet from the server.\n\nSynonyms:\ngemqtt.set_auto_ack\ngemqtt set_auto_ack","url":"gemqtt.html#set_auto_ack"},{"doc":"gemqtt","title":"set_clean_start","content":"pub fn set_clean_start(opts: Options, clean: Bool) -&gt; Options\n Controls whether the server should discard any existing session and start a\n new session, can prevent receiving old retained messages. Defaults to true.\n\nSynonyms:\ngemqtt.set_clean_start\ngemqtt set_clean_start","url":"gemqtt.html#set_clean_start"},{"doc":"gemqtt","title":"set_client_id","content":"pub fn set_client_id(opts: Options, id: String) -&gt; Options\n Specifies the client identifier. If not given, the client identifier be\n assigned by the server in MQTT v5 or be automatically generated by _emqtt_\n in MQTT v3.1/v3.1.1.\n\nSynonyms:\ngemqtt.set_client_id\ngemqtt set_client_id","url":"gemqtt.html#set_client_id"},{"doc":"gemqtt","title":"set_connect_timeout","content":"pub fn set_connect_timeout(\n  opts: Options,\n  seconds timeout: Int,\n) -&gt; Options\n Sets the maximum time to wait to connect to the server and the server\n returns a CONNACK. Defaults to 60s.\n\nSynonyms:\ngemqtt.set_connect_timeout\ngemqtt set_connect_timeout","url":"gemqtt.html#set_connect_timeout"},{"doc":"gemqtt","title":"set_name","content":"pub fn set_name(opts: Options, name: String) -&gt; Options\n Sets the Erlang server name for the client process.\n\nSynonyms:\ngemqtt.set_name\ngemqtt set_name","url":"gemqtt.html#set_name"},{"doc":"gemqtt","title":"set_owner","content":"pub fn set_owner(opts: Options, pid: Pid) -&gt; Options\n Specifies the owner, to which the client process will send messages like\n `{disconnected, ReasonCode, Properties}`.\n\nSynonyms:\ngemqtt.set_owner\ngemqtt set_owner","url":"gemqtt.html#set_owner"},{"doc":"gemqtt","title":"set_port","content":"pub fn set_port(opts: Options, port: Int) -&gt; Options\n Specifies port of the MQTT server to be connected. If not given, the\n default of 1883 for MQTT or 8883 for MQTT over TLS will be used.\n\nSynonyms:\ngemqtt.set_port\ngemqtt set_port","url":"gemqtt.html#set_port"},{"doc":"gemqtt","title":"set_property","content":"pub fn set_property(\n  opts: Options,\n  name: String,\n  value: a,\n) -&gt; Options\n Sets an MQTT CONNECT packet property.  Please note that properties are\n validated when `start_link` is called; if _emqtt_ detects errors it will\n cause your process to fail rather than return an Error.\n\n Available connect properties:\n\n - `Session-Expiry-Interval`: Four byte integer\n - `Receive-Maximum`: Two byte integer\n - `Maximum-Packet-Size`: Four byte integer\n - `Topic-Alias-Maximum`: Two byte integer\n - `Request-Response-Information`: Byte\n - `Request-Problem-Information`: Byte\n - `User-Property`: UTF-8 string pair\n - `Authentication-Method`: UTF-8 encoded string\n - `Authentication-Data`: Binary data\n\n Example usage:\n\n \n gemqtt.new(&quot;localhost&quot;)\n |&gt; gemqtt.set_property(&quot;Maximum-Packet-Size&quot;, 300)\n |&gt; gemqtt.set_property(&quot;User-Property&quot;, #(&quot;prop-name&quot;, &quot;prop-value&quot;))\n \n\nSynonyms:\ngemqtt.set_property\ngemqtt set_property","url":"gemqtt.html#set_property"},{"doc":"gemqtt","title":"set_tls_enabled","content":"pub fn set_tls_enabled(opts: Options, tls: Bool) -&gt; Options\n Enables SSL/TLS transport encryption. Defaults to false.\n\nSynonyms:\ngemqtt.set_tls_enabled\ngemqtt set_tls_enabled","url":"gemqtt.html#set_tls_enabled"},{"doc":"gemqtt","title":"set_tls_opt","content":"pub fn set_tls_opt(\n  opts: Options,\n  name: String,\n  value: Dynamic,\n) -&gt; Options\n Primitive function for setting TLS related options. Needs improvement.\n\n Example usage:\n\n     fn main() {\n       gemqtt.new(&quot;mqtt.example.com&quot;)\n       |&gt; gemqtt.set_tls_enabled(True)\n       |&gt; gemqtt.set_tls_opt(\n         &quot;verify&quot;,\n         dynamic.from(atom.create_from_string(&quot;verify_peer&quot;)),\n       )\n       |&gt; gemqtt.set_tls_opt(\n         &quot;server_name_indication&quot;,\n         dynamic.from(atom.create_from_string(&quot;disable&quot;)),\n       )\n       |&gt; gemqtt.set_tls_opt(&quot;cacerts&quot;, cacerts_get())\n     }\n\n     // Returns the OS cacerts.\n     @external(erlang, &quot;public_key&quot;, &quot;cacerts_get&quot;)\n     fn cacerts_get() -&gt; Dynamic\n\nSynonyms:\ngemqtt.set_tls_opt\ngemqtt set_tls_opt","url":"gemqtt.html#set_tls_opt"},{"doc":"gemqtt","title":"start_link","content":"pub fn start_link(opts: Options) -&gt; Result(Client, Error)\n Configures a client process and link it to ours.  Does not attempt to\n connect to the MQTT server.\n\nSynonyms:\ngemqtt.start_link\ngemqtt start_link","url":"gemqtt.html#start_link"},{"doc":"gemqtt","title":"stop","content":"pub fn stop(client: Client) -&gt; Result(Nil, Error)\n Stops the client process, prefer `disconnect` over this.\n\nSynonyms:\ngemqtt.stop\ngemqtt stop","url":"gemqtt.html#stop"},{"doc":"gemqtt","title":"subscriptions","content":"pub fn subscriptions(\n  client: Client,\n) -&gt; List(#(String, Dict(Atom, Dynamic)))\n Lists this clients active MQTT subscriptions.\n\nSynonyms:\ngemqtt.subscriptions\ngemqtt subscriptions","url":"gemqtt.html#subscriptions"},{"doc":"gemqtt","title":"gemqtt","content":" gemqtt provides an MQTT client for Gleam projects running on the BEAM.  It\n does so by wrapping the [emqtt](https://github.com/emqx/emqtt) library\n written in Erlang.\n\n The `gemqtt` package contains functions for configuring and controlling\n the client process.  See the `publisher` and `subscriber` packages for\n working with MQTT messages.\n\n Example client setup:\n\n     let assert Ok(client) =\n       gemqtt.new(&quot;test.mosquitto.org&quot;)\n       |&gt; gemqtt.set_auth(&quot;user&quot;, &quot;password&quot;)\n       |&gt; gemqtt.start_link\n\n     let assert Ok(_) = gemqtt.connect(client)","url":"gemqtt.html"},{"doc":"gemqtt/publisher","title":"PublishOption","content":"pub type PublishOption {\n  Retain(Bool)\n  Qos(gemqtt.Qos)\n}\n\nRetain(Bool)\n\n\nQos(gemqtt.Qos)\n\n\nSynonyms:\ngemqtt/publisher.PublishOption\ngemqtt/publisher PublishOption","url":"gemqtt/publisher.html#PublishOption"},{"doc":"gemqtt/publisher","title":"Publisher","content":"pub type Publisher {\n  Publisher(\n    client: Client,\n    topic: String,\n    options: List(PublishOption),\n    properties: Properties,\n  )\n}\n Publisher holds the configuration for publishing to a particular topic.\n Create it with the `new` function.\n\nPublisher(\n  client: Client,\n  topic: String,\n  options: List(PublishOption),\n  properties: Properties,\n)\n\n\nSynonyms:\ngemqtt/publisher.Publisher\ngemqtt/publisher Publisher","url":"gemqtt/publisher.html#Publisher"},{"doc":"gemqtt/publisher","title":"new","content":"pub fn new(client: Client, topic: String) -&gt; Publisher\n Creates a new publisher for the specified topic.\n\nSynonyms:\ngemqtt/publisher.new\ngemqtt/publisher new","url":"gemqtt/publisher.html#new"},{"doc":"gemqtt/publisher","title":"publish","content":"pub fn publish(\n  publisher: Publisher,\n  payload: BitArray,\n) -&gt; Result(Option(Int), Error)\n Sends a PUBLISH packet to the server.\n\n Upon success, will returns `Ok(Some(packet ID))` if QoS is `AtLeastOnce` or\n `ExactlyOnce`, otherwise `Ok(None)`.\n\nSynonyms:\ngemqtt/publisher.publish\ngemqtt/publisher publish","url":"gemqtt/publisher.html#publish"},{"doc":"gemqtt/publisher","title":"set_qos","content":"pub fn set_qos(publisher: Publisher, qos: Qos) -&gt; Publisher\n Sets the QoS level for published messages; defaults to `AtMostOnce`.\n\nSynonyms:\ngemqtt/publisher.set_qos\ngemqtt/publisher set_qos","url":"gemqtt/publisher.html#set_qos"},{"doc":"gemqtt/publisher","title":"set_retain","content":"pub fn set_retain(\n  publisher: Publisher,\n  retain: Bool,\n) -&gt; Publisher\n Indicates to the server whether the most recently published message should\n be retained for future subscribers as a _last known good_ message on this\n topic.  Defaults to false.\n\nSynonyms:\ngemqtt/publisher.set_retain\ngemqtt/publisher set_retain","url":"gemqtt/publisher.html#set_retain"},{"doc":"gemqtt/publisher","title":"gemqtt/publisher","content":" Module for publishing messages to an MQTT server.\n\n Example usage:\n\n     let msg_payload = bit_array.from_string(&quot;test payload&quot;)\n\n     let assert Ok(_) =\n       publisher.new(client, &quot;my/topic&quot;)\n       |&gt; publisher.set_qos(gemqtt.AtLeastOnce)\n       |&gt; publisher.publish(msg_payload)","url":"gemqtt/publisher.html"},{"doc":"gemqtt/subscriber","title":"Message","content":"pub type Message {\n  Message(\n    client: Client,\n    duplicate: Bool,\n    packet_id: Option(Int),\n    payload: BitArray,\n    qos: gemqtt.Qos,\n    retain: Bool,\n    topic: String,\n  )\n}\n Representation of a message received from an MQTT topic.\n\nMessage(\n  client: Client,\n  duplicate: Bool,\n  packet_id: Option(Int),\n  payload: BitArray,\n  qos: gemqtt.Qos,\n  retain: Bool,\n  topic: String,\n)\n\n\nSynonyms:\ngemqtt/subscriber.Message\ngemqtt/subscriber Message","url":"gemqtt/subscriber.html#Message"},{"doc":"gemqtt/subscriber","title":"PubAck","content":"pub type PubAck {\n  PubAck(packet_id: Int, reason_code: Int)\n}\n Packet sent by MQTT server to acknowledge a published message.\n\nPubAck(packet_id: Int, reason_code: Int)\n\n\nSynonyms:\ngemqtt/subscriber.PubAck\ngemqtt/subscriber PubAck","url":"gemqtt/subscriber.html#PubAck"},{"doc":"gemqtt/subscriber","title":"RetainHandling","content":"pub type RetainHandling {\n  SentAlways\n  SentOnNewSubscription\n  SentNever\n}\n Represents how retained messages on a topic are handled when a subscription\n to it is added.\n\nSentAlways\n Retained messages are sent whenever a subscription is established.\n\nSentOnNewSubscription\n Retained messages are sent only when establishing a new subscription,\n not a repeated one.\n\nSentNever\n No retained messages are sent when a subscription is established.\n\nSynonyms:\ngemqtt/subscriber.RetainHandling\ngemqtt/subscriber RetainHandling","url":"gemqtt/subscriber.html#RetainHandling"},{"doc":"gemqtt/subscriber","title":"Subscriber","content":"pub opaque type Subscriber\n Subscriber holds the configuration for subscribing to one or more topics.\n Create it with the `new` function.\n\n\nSynonyms:\ngemqtt/subscriber.Subscriber\ngemqtt/subscriber Subscriber","url":"gemqtt/subscriber.html#Subscriber"},{"doc":"gemqtt/subscriber","title":"add","content":"pub fn add(\n  sub: Subscriber,\n  topics topics: List(String),\n) -&gt; Result(#(Option(Properties), List(Int)), Nil)\n\nSynonyms:\ngemqtt/subscriber.add\ngemqtt/subscriber add","url":"gemqtt/subscriber.html#add"},{"doc":"gemqtt/subscriber","title":"message_from_dynamic","content":"pub fn message_from_dynamic(\n  input: Dynamic,\n) -&gt; Result(Message, List(DecodeError))\n Decodes a message received by emqtt. Prefer `selecting_mqtt_messages`\n over using this directly.\n\nSynonyms:\ngemqtt/subscriber.message_from_dynamic\ngemqtt/subscriber message_from_dynamic","url":"gemqtt/subscriber.html#message_from_dynamic"},{"doc":"gemqtt/subscriber","title":"new","content":"pub fn new(client: Client) -&gt; Subscriber\n Creates a new subscriber with the default options.  Defaults are documented\n on each set function.\n\nSynonyms:\ngemqtt/subscriber.new\ngemqtt/subscriber new","url":"gemqtt/subscriber.html#new"},{"doc":"gemqtt/subscriber","title":"puback_from_dynamic","content":"pub fn puback_from_dynamic(\n  input: Dynamic,\n) -&gt; Result(PubAck, List(DecodeError))\n Decodes a pub-ack received by emqtt. Prefer `selecting_mqtt_pubacks`\n over using this directly.\n\nSynonyms:\ngemqtt/subscriber.puback_from_dynamic\ngemqtt/subscriber puback_from_dynamic","url":"gemqtt/subscriber.html#puback_from_dynamic"},{"doc":"gemqtt/subscriber","title":"remove","content":"pub fn remove(\n  client: Client,\n  topics topics: List(String),\n) -&gt; Result(#(Option(Properties), List(Int)), Nil)\n\nSynonyms:\ngemqtt/subscriber.remove\ngemqtt/subscriber remove","url":"gemqtt/subscriber.html#remove"},{"doc":"gemqtt/subscriber","title":"selecting_mqtt_messages","content":"pub fn selecting_mqtt_messages(\n  selector: Selector(a),\n  mapper: fn(Message) -&gt; a,\n) -&gt; Selector(a)\n Configures a selector to receive messages from MQTT clients.\n\n Please note this will receive messages from all MQTT clients that the\n process controls, rather than any specific one. If you wish to only handle\n messages from one client then use one process per client.\n\nSynonyms:\ngemqtt/subscriber.selecting_mqtt_messages\ngemqtt/subscriber selecting_mqtt_messages","url":"gemqtt/subscriber.html#selecting_mqtt_messages"},{"doc":"gemqtt/subscriber","title":"selecting_mqtt_pubacks","content":"pub fn selecting_mqtt_pubacks(\n  selector: Selector(a),\n  mapper: fn(PubAck) -&gt; a,\n) -&gt; Selector(a)\n Configures a selector to receive pub-acks from MQTT clients.\n\n Please note this will receive pub-acks from all MQTT clients that the\n process controls, rather than any specific one.\n\nSynonyms:\ngemqtt/subscriber.selecting_mqtt_pubacks\ngemqtt/subscriber selecting_mqtt_pubacks","url":"gemqtt/subscriber.html#selecting_mqtt_pubacks"},{"doc":"gemqtt/subscriber","title":"set_local_echo","content":"pub fn set_local_echo(\n  subscriber: Subscriber,\n  value: Bool,\n) -&gt; Subscriber\n Controls whether a subscriber will see messages published by the same\n client ID as itself; defaults to true.\n\n Warning: doesn&#39;t seem to work correctly, possibly an emqtt or mosquitto\n issue.\n\nSynonyms:\ngemqtt/subscriber.set_local_echo\ngemqtt/subscriber set_local_echo","url":"gemqtt/subscriber.html#set_local_echo"},{"doc":"gemqtt/subscriber","title":"set_property","content":"pub fn set_property(\n  sub: Subscriber,\n  name: String,\n  value: a,\n) -&gt; Subscriber\n Sets an MQTT SUBSCRIBE packet property.  Please note that properties are\n not validated prior to being sent to the server.\n\n Available properties:\n\n - `Subscription-Identifier`: 1 to 268,435,455\n - `User-Property`: UTF-8 string pair\n\n Example usage:\n\n     client\n     |&gt; subscriber.new\n     |&gt; subscriber.set_property(&quot;Subscription-Identifier&quot;, 42)\n     |&gt; subscriber.set_property(&quot;User-Property&quot;, #(&quot;prop-name&quot;, &quot;prop-value&quot;))\n\nSynonyms:\ngemqtt/subscriber.set_property\ngemqtt/subscriber set_property","url":"gemqtt/subscriber.html#set_property"},{"doc":"gemqtt/subscriber","title":"set_qos","content":"pub fn set_qos(subscriber: Subscriber, value: Qos) -&gt; Subscriber\n Specifies the highest QoS level for topics on this subscriber.\n\n (low) `AtMostOnce` &lt; `AtLeastOnce` &lt; `ExactlyOnce` (high).\n\n Defaults to `AtMostOnce`.\n\nSynonyms:\ngemqtt/subscriber.set_qos\ngemqtt/subscriber set_qos","url":"gemqtt/subscriber.html#set_qos"},{"doc":"gemqtt/subscriber","title":"set_retain_as_published","content":"pub fn set_retain_as_published(\n  subscriber: Subscriber,\n  value: Bool,\n) -&gt; Subscriber\n Controls whether the subscriptions will have the &quot;retain as published&quot;\n option set. When true, the retain flag on an incoming message will be\n exactly as set by the publishing client, rather than indicating whether the\n message is fresh/stale.\n\n Defaults to false.\n\nSynonyms:\ngemqtt/subscriber.set_retain_as_published\ngemqtt/subscriber set_retain_as_published","url":"gemqtt/subscriber.html#set_retain_as_published"},{"doc":"gemqtt/subscriber","title":"set_retain_handling","content":"pub fn set_retain_handling(\n  subscriber: Subscriber,\n  value: RetainHandling,\n) -&gt; Subscriber\n Controls how retained messages are handled when a subscription is added.\n See the documentaton on the `RetainHandling` type for more information.\n\n Defaults to `SentAlways`.\n\nSynonyms:\ngemqtt/subscriber.set_retain_handling\ngemqtt/subscriber set_retain_handling","url":"gemqtt/subscriber.html#set_retain_handling"},{"doc":"gemqtt/subscriber","title":"gemqtt/subscriber","content":" Module for subscribing to messages from an MQTT server.\n\n Example usage:\n\n     client\n     |&gt; subscriber.new\n     |&gt; subscriber.set_qos(gemqtt.AtLestOnce)\n     |&gt; subscriber.add([&quot;my/topic&quot;, &quot;another/topic&quot;])\n\n     let assert Ok(got_msg) =\n       process.new_selector()\n       |&gt; subscriber.selecting_mqtt_messages(Ok)\n       |&gt; process.select_forever","url":"gemqtt/subscriber.html"}]);